@@ -1,15 +1,15 @@
 use core::pin::Pin;
 
 use futures::future::FutureExt;
-use futures::io::{AsyncRead, Error, ErrorKind};
+use futures::io::{AsyncRead, Error};
 use futures::ready;
 use futures::task::{Context, Poll};
-use js_sys::{Object, Uint8Array};
+use js_sys::Uint8Array;
 use wasm_bindgen::prelude::*;
 use wasm_bindgen::JsCast;
 use wasm_bindgen_futures::JsFuture;
 
-use crate::util::{checked_cast_to_usize, clamp_to_u32};
+use crate::util::{checked_cast_to_usize, clamp_to_u32, js_to_io_error};
 
 use super::sys::{ArrayBufferView, ReadableStreamBYOBReadResult};
 use super::ReadableStreamBYOBReader;
@@ -27,7 +27,7 @@ use super::ReadableStreamBYOBReader;
 /// it is up to the user to either manually [cancel](Self::cancel) the stream,
 /// or to ensure that there are no pending read requests when dropped.
 /// See the documentation on [`ReadableStreamBYOBReader`] for more details on the drop behavior.
-#[must_use = "streams do nothing unless polled"]
+#[must_use = "readers do nothing unless polled"]
 #[derive(Debug)]
 pub struct IntoAsyncRead<'reader> {
     reader: Option<ReadableStreamBYOBReader<'reader>>,
@@ -78,48 +78,50 @@ impl<'reader> AsyncRead for IntoAsyncRead<'reader> {
         cx: &mut Context<'_>,
         buf: &mut [u8],
     ) -> Poll<Result<usize, Error>> {
-        if self.fut.is_none() {
-            // No pending read, start reading the next bytes
-            let buf_len = clamp_to_u32(buf.len());
-            let buffer = match self.buffer.take() {
-                // Re-use the internal buffer if it is large enough,
-                // otherwise allocate a new one
-                Some(buffer) if buffer.byte_length() >= buf_len => buffer,
-                _ => Uint8Array::new_with_length(buf_len),
-            };
-            // Limit to output buffer size
-            let buffer = buffer
-                .subarray(0, buf_len)
-                .unchecked_into::<ArrayBufferView>();
-            match &self.reader {
-                Some(reader) => {
-                    // Read into internal buffer and store its future
-                    let fut = JsFuture::from(reader.as_raw().read(&buffer));
-                    self.fut = Some(fut);
-                }
-                None => {
-                    // Reader was already dropped
-                    return Poll::Ready(Ok(0));
+        let read_fut = match self.fut.as_mut() {
+            Some(fut) => fut,
+            None => {
+                // No pending read, start reading the next bytes
+                let buf_len = clamp_to_u32(buf.len());
+                let buffer = match self.buffer.take() {
+                    // Re-use the internal buffer if it is large enough,
+                    // otherwise allocate a new one
+                    Some(buffer) if buffer.byte_length() >= buf_len => buffer,
+                    _ => Uint8Array::new_with_length(buf_len),
+                };
+                // Limit to output buffer size
+                let buffer = buffer
+                    .subarray(0, buf_len)
+                    .unchecked_into::<ArrayBufferView>();
+                match &self.reader {
+                    Some(reader) => {
+                        // Read into internal buffer and store its future
+                        let fut = JsFuture::from(reader.as_raw().read(&buffer));
+                        self.fut.insert(fut)
+                    }
+                    None => {
+                        // Reader was already dropped
+                        return Poll::Ready(Ok(0));
+                    }
                 }
             }
-        }
+        };
 
         // Poll the future for the pending read
-        let js_result = ready!(self.as_mut().fut.as_mut().unwrap_throw().poll_unpin(cx));
+        let js_result = ready!(read_fut.poll_unpin(cx));
         self.fut = None;
 
         // Read completed
         Poll::Ready(match js_result {
             Ok(js_value) => {
                 let result = ReadableStreamBYOBReadResult::from(js_value);
-                let filled_view = result.value();
                 if result.is_done() {
                     // End of stream
                     self.discard_reader();
                     Ok(0)
                 } else {
                     // Cannot be canceled, so view must exist
-                    let filled_view = filled_view.unwrap_throw();
+                    let filled_view = result.value().unwrap_throw();
                     // Copy bytes to output buffer
                     let filled_len = checked_cast_to_usize(filled_view.byte_length());
                     debug_assert!(filled_len <= buf.len());
@@ -132,11 +134,7 @@ impl<'reader> AsyncRead for IntoAsyncRead<'reader> {
             Err(js_value) => {
                 // Error
                 self.discard_reader();
-                let error = match Object::try_from(&js_value) {
-                    Some(js_object) => js_object.to_string().as_string().unwrap_throw(),
-                    None => "Unknown error".to_string(),
-                };
-                Err(Error::new(ErrorKind::Other, error))
+                Err(js_to_io_error(js_value))
             }
         })
     }
