@@ -4,7 +4,7 @@ use wasm_bindgen::JsValue;
 
 use crate::util::promise_to_void_future;
 
-use super::{sys, IntoSink, WritableStream};
+use super::{sys, IntoAsyncWrite, IntoSink, WritableStream};
 
 /// A [`WritableStreamDefaultWriter`](https://developer.mozilla.org/en-US/docs/Web/API/WritableStreamDefaultWriter)
 /// that can be used to write chunks to a [`WritableStream`](WritableStream).
@@ -118,6 +118,19 @@ impl<'stream> WritableStreamDefaultWriter<'stream> {
     pub fn into_sink(self) -> IntoSink<'stream> {
         IntoSink::new(self)
     }
+
+    /// Converts this `WritableStreamDefaultWriter` into an [`AsyncWrite`](futures::AsyncWrite).
+    ///
+    /// The writable stream must accept [`Uint8Array`](js_sys::Uint8Array) chunks.
+    ///
+    /// This is similar to [`WritableStream.into_async_write`](WritableStream::into_async_write),
+    /// except that after the returned `AsyncWrite` is dropped, the original `WritableStream` is
+    /// still usable. This allows writing only a few bytes through the `AsyncWrite`, while still
+    /// allowing another writer to write more bytes later on.
+    #[inline]
+    pub fn into_async_write(self) -> IntoAsyncWrite<'stream> {
+        IntoAsyncWrite::new(self.into_sink())
+    }
 }
 
 impl Drop for WritableStreamDefaultWriter<'_> {
